@@ -32,6 +32,33 @@ module Vagrant
         end
 
         def call(env)
+          @leased  = []
+          @machine = env[:machine]
+
+          # Acquire a process-level lock so that we don't choose a port
+          # that someone else also chose.
+          begin
+            env[:machine].env.lock("fpcollision") do
+              handle(env)
+            end
+          rescue Errors::EnvironmentLockedError
+            sleep 1
+            retry
+          end
+
+          @app.call(env)
+
+          # Always run the recover method so that we release leases
+          recover(env)
+        end
+
+        def recover(env)
+          lease_release
+        end
+
+        protected
+
+        def handle(env)
           @logger.info("Detecting any forwarded port collisions...")
 
           # Get the extra ports we consider in use
@@ -43,6 +70,10 @@ module Vagrant
           # Determine the handler we'll use if we have any port collisions
           repair = !!env[:port_collision_repair]
 
+          # The method we'll use to check if a port is open.
+          port_checker = env[:port_collision_port_check]
+          port_checker ||= method(:port_check)
+
           # Log out some of our parameters
           @logger.debug("Extra in use: #{extra_in_use.inspect}")
           @logger.debug("Remap: #{remap.inspect}")
@@ -62,6 +93,11 @@ module Vagrant
             guest_port = options[:guest]
             host_port  = options[:host]
 
+            if options[:protocol] && options[:protocol] != "tcp"
+              @logger.debug("Skipping #{host_port} because UDP protocol.")
+              next
+            end
+
             if remap[host_port]
               remap_port = remap[host_port]
               @logger.debug("Remap port override: #{host_port} => #{remap_port}")
@@ -69,11 +105,14 @@ module Vagrant
             end
 
             # If the port is open (listening for TCP connections)
-            if extra_in_use.include?(host_port) || is_port_open?("127.0.0.1", host_port)
+            in_use = extra_in_use.include?(host_port) ||
+              port_checker[host_port] ||
+              lease_check(host_port)
+            if in_use
               if !repair || !options[:auto_correct]
                 raise Errors::ForwardPortCollision,
-                  :guest_port => guest_port.to_s,
-                  :host_port  => host_port.to_s
+                  guest_port: guest_port.to_s,
+                  host_port:  host_port.to_s
               end
 
               @logger.info("Attempting to repair FP collision: #{host_port}")
@@ -85,7 +124,10 @@ module Vagrant
                 usable_ports.delete(repaired_port)
 
                 # If the port is in use, then we can't use this either...
-                if extra_in_use.include?(repaired_port) || is_port_open?("127.0.0.1", repaired_port)
+                in_use = extra_in_use.include?(repaired_port) ||
+                  port_checker[repaired_port] ||
+                  lease_check(repaired_port)
+                if in_use
                   @logger.info("Reparied port also in use: #{repaired_port}. Trying another...")
                   next
                 end
@@ -97,9 +139,9 @@ module Vagrant
               # If we have no usable ports then we can't repair
               if !repaired_port && usable_ports.empty?
                 raise Errors::ForwardPortAutolistEmpty,
-                  :vm_name    => env[:machine].name,
-                  :guest_port => guest_port.to_s,
-                  :host_port  => host_port.to_s
+                  vm_name:    env[:machine].name,
+                  guest_port: guest_port.to_s,
+                  host_port:  host_port.to_s
               end
 
               # Modify the args in place
@@ -109,16 +151,63 @@ module Vagrant
 
               # Notify the user
               env[:ui].info(I18n.t("vagrant.actions.vm.forward_ports.fixed_collision",
-                                   :host_port  => host_port.to_s,
-                                   :guest_port => guest_port.to_s,
-                                   :new_port   => repaired_port.to_s))
+                                   host_port:  host_port.to_s,
+                                   guest_port: guest_port.to_s,
+                                   new_port:   repaired_port.to_s))
             end
           end
 
           @app.call(env)
         end
 
-        protected
+        def lease_check(port)
+          # Check if this port is "leased". We use a leasing system of
+          # about 60 seconds to avoid any forwarded port collisions in
+          # a highly parallelized environment.
+          leasedir = @machine.env.data_dir.join("fp-leases")
+          leasedir.mkpath
+
+          invalid = false
+          oldest  = Time.now.to_i - 60
+          leasedir.children.each do |child|
+            # Delete old, invalid leases while we're looking
+            if child.file? && child.mtime.to_i < oldest
+              child.delete
+            end
+
+            if child.basename.to_s == port.to_s
+              invalid = true
+            end
+          end
+
+          # If its invalid, then the port is "open" and in use
+          return true if invalid
+
+          # Otherwise, create the lease
+          leasedir.join(port.to_s).open("w+") do |f|
+            f.binmode
+            f.write(Time.now.to_i.to_s + "\n")
+          end
+
+          # Add to the leased array so we unlease it right away
+          @leased << port.to_s
+
+          # Things look good to us!
+          false
+        end
+
+        def lease_release
+          leasedir = @machine.env.data_dir.join("fp-leases")
+
+          @leased.each do |port|
+            path = leasedir.join(port)
+            path.delete if path.file?
+          end
+        end
+
+        def port_check(port)
+          is_port_open?("127.0.0.1", port)
+        end
 
         def with_forwarded_ports(env)
           env[:machine].config.vm.networks.each do |type, options|
