@@ -5,7 +5,7 @@ module Vagrant
         # This returns all the instances of the configured provisioners.
         # This is safe to call multiple times since it will cache the results.
         #
-        # @return [Array<Provisioner>]
+        # @return [Array<Provisioner, Hash>]
         def provisioner_instances(env)
           return @_provisioner_instances if @_provisioner_instances
 
@@ -16,16 +16,25 @@ module Vagrant
           @_provisioner_instances = env[:machine].config.vm.provisioners.map do |provisioner|
             # Instantiate the provisioner
             klass  = Vagrant.plugin("2").manager.provisioners[provisioner.name]
+
+            # This can happen in the case the configuration isn't validated.
+            next nil if !klass
+
             result = klass.new(env[:machine], provisioner.config)
 
             # Store in the type map so that --provision-with works properly
             @_provisioner_types[result] = provisioner.name
 
+            # Build up the options
+            options = {
+              run: provisioner.run,
+            }
+
             # Return the result
-            result
+            [result, options]
           end
 
-          return @_provisioner_instances
+          return @_provisioner_instances.compact
         end
 
         # This will return a mapping of a provisioner instance to its
