@@ -21,18 +21,23 @@ module Vagrant
         end
 
         def call(env)
-          # Grab the SSH info from the machine
-          info = env[:machine].ssh_info
+          # Grab the SSH info from the machine or the environment
+          info = env[:ssh_info]
+          info ||= env[:machine].ssh_info
 
           # If the result is nil, then the machine is telling us that it is
           # not yet ready for SSH, so we raise this exception.
           raise Errors::SSHNotReady if info.nil?
 
-          if info[:private_key_path]
-            # Check SSH key permissions
-            info[:private_key_path].each do |path|
-              SSH.check_key_permissions(Pathname.new(path))
-            end
+          info[:private_key_path] ||= []
+
+          # Check SSH key permissions
+          info[:private_key_path].each do |path|
+            SSH.check_key_permissions(Pathname.new(path))
+          end
+
+          if info[:private_key_path].empty? && info[:password]
+            env[:ui].warn(I18n.t("vagrant.ssh_exec_password"))
           end
 
           # Exec!
