@@ -1,5 +1,6 @@
 require "log4r"
 
+require "vagrant/util/platform"
 require "vagrant/util/ssh"
 require "vagrant/util/shell_quote"
 
@@ -26,11 +27,15 @@ module Vagrant
           # not yet ready for SSH, so we raise this exception.
           raise Errors::SSHNotReady if info.nil?
 
-          if info[:private_key_path]
-            # Check SSH key permissions
-            info[:private_key_path].each do |path|
-              SSH.check_key_permissions(Pathname.new(path))
-            end
+          info[:private_key_path] ||= []
+
+          # Check SSH key permissions
+          info[:private_key_path].each do |path|
+            SSH.check_key_permissions(Pathname.new(path))
+          end
+
+          if info[:private_key_path].empty?
+            raise Errors::SSHRunRequiresKeys
           end
 
           # Get the command and wrap it in a login shell
@@ -39,6 +44,7 @@ module Vagrant
 
           # Execute!
           opts = env[:ssh_opts] || {}
+          opts[:extra_args] ||= []
 
           # Allow the user to specify a tty or non-tty manually, but if they
           # don't then we default to a TTY
