@@ -76,7 +76,6 @@ module Vagrant
 
         unknown_sources = @sources.keys - order
         if !unknown_sources.empty?
-          # TODO: Raise exception here perhaps.
           @logger.error("Unknown config sources: #{unknown_sources.inspect}")
         end
 
@@ -198,7 +197,7 @@ module Vagrant
             Kernel.load path
           rescue SyntaxError => e
             # Report syntax errors in a nice way.
-            raise Errors::VagrantfileSyntaxError, :file => e.message
+            raise Errors::VagrantfileSyntaxError, file: e.message
           rescue SystemExit
             # Continue raising that exception...
             raise
@@ -212,8 +211,8 @@ module Vagrant
 
             # Report the generic exception
             raise Errors::VagrantfileLoadError,
-              :path => path,
-              :message => e.message
+              path: path,
+              message: e.message
           end
         end
       end
