@@ -43,6 +43,9 @@ module Vagrant
     # error code, and the error key is used as a default message from
     # I18n.
     class VagrantError < StandardError
+      # This is extra data passed into the message for translation.
+      attr_accessor :extra_data
+
       def self.error_key(key=nil, namespace=nil)
         define_method(:error_key) { key }
         error_namespace(namespace) if namespace
@@ -56,9 +59,14 @@ module Vagrant
         define_method(:error_namespace) { namespace }
       end
 
-      def initialize(message=nil, *args)
-        message = { :_key => message } if message && !message.is_a?(Hash)
-        message = { :_key => error_key, :_namespace => error_namespace }.merge(message || {})
+      def initialize(*args)
+        key     = args.shift if args.first.is_a?(Symbol)
+        message = args.shift if args.first.is_a?(Hash)
+        message ||= {}
+        @extra_data    = message.dup
+        message[:_key] ||= error_key
+        message[:_namespace] ||= error_namespace
+        message[:_key] = key if key
 
         if message[:_key]
           message = translate_error(message)
@@ -66,7 +74,7 @@ module Vagrant
           message = error_message
         end
 
-        super
+        super(message)
       end
 
       # The error message for this error. This is used if no error_key
@@ -108,16 +116,40 @@ module Vagrant
       error_key(:ansible_playbook_app_not_found)
     end
 
-    class BaseVMNotFound < VagrantError
-      error_key(:base_vm_not_found)
-    end
-
     class BatchMultiError < VagrantError
       error_key(:batch_multi_error)
     end
 
+    class BoxAddDirectVersion < VagrantError
+      error_key(:box_add_direct_version)
+    end
+
+    class BoxAddMetadataMultiURL < VagrantError
+      error_key(:box_add_metadata_multi_url)
+    end
+
+    class BoxAddNameMismatch < VagrantError
+      error_key(:box_add_name_mismatch)
+    end
+
+    class BoxAddNameRequired < VagrantError
+      error_key(:box_add_name_required)
+    end
+
+    class BoxAddNoMatchingProvider < VagrantError
+      error_key(:box_add_no_matching_provider)
+    end
+
+    class BoxAddNoMatchingVersion < VagrantError
+      error_key(:box_add_no_matching_version)
+    end
+
+    class BoxAddShortNotFound < VagrantError
+      error_key(:box_add_short_not_found)
+    end
+
     class BoxAlreadyExists < VagrantError
-      error_key(:already_exists, "vagrant.actions.box.unpackage")
+      error_key(:box_add_exists)
     end
 
     class BoxChecksumInvalidType < VagrantError
@@ -136,38 +168,102 @@ module Vagrant
       error_key(:box_metadata_corrupted)
     end
 
+    class BoxMetadataDownloadError < VagrantError
+      error_key(:box_metadata_download_error)
+    end
+
     class BoxMetadataFileNotFound < VagrantError
       error_key(:box_metadata_file_not_found)
     end
 
+    class BoxMetadataMalformed < VagrantError
+      error_key(:box_metadata_malformed)
+    end
+
+    class BoxMetadataMalformedVersion < VagrantError
+      error_key(:box_metadata_malformed_version)
+    end
+
     class BoxNotFound < VagrantError
       error_key(:box_not_found)
     end
 
-    class BoxNotSpecified < VagrantError
-      error_key(:not_specified, "vagrant.actions.vm.check_box")
+    class BoxNotFoundWithProvider < VagrantError
+      error_key(:box_not_found_with_provider)
+    end
+
+    class BoxNotFoundWithProviderAndVersion < VagrantError
+      error_key(:box_not_found_with_provider_and_version)
     end
 
     class BoxProviderDoesntMatch < VagrantError
       error_key(:box_provider_doesnt_match)
     end
 
-    class BoxSpecifiedDoesntExist < VagrantError
-      error_key(:does_not_exist, "vagrant.actions.vm.check_box")
+    class BoxRemoveNotFound < VagrantError
+      error_key(:box_remove_not_found)
+    end
+
+    class BoxRemoveProviderNotFound < VagrantError
+      error_key(:box_remove_provider_not_found)
+    end
+
+    class BoxRemoveVersionNotFound < VagrantError
+      error_key(:box_remove_version_not_found)
+    end
+
+    class BoxRemoveMultiProvider < VagrantError
+      error_key(:box_remove_multi_provider)
+    end
+
+    class BoxRemoveMultiVersion < VagrantError
+      error_key(:box_remove_multi_version)
+    end
+
+    class BoxServerNotSet < VagrantError
+      error_key(:box_server_not_set)
     end
 
     class BoxUnpackageFailure < VagrantError
       error_key(:untar_failure, "vagrant.actions.box.unpackage")
     end
 
-    class BoxUpgradeRequired < VagrantError
-      error_key(:box_upgrade_required)
+    class BoxUpdateMultiProvider < VagrantError
+      error_key(:box_update_multi_provider)
+    end
+
+    class BoxUpdateNoMetadata < VagrantError
+      error_key(:box_update_no_metadata)
     end
 
     class BoxVerificationFailed < VagrantError
       error_key(:failed, "vagrant.actions.box.verify")
     end
 
+    class BundlerDisabled < VagrantError
+      error_key(:bundler_disabled)
+    end
+
+    class BundlerError < VagrantError
+      error_key(:bundler_error)
+    end
+
+    class CapabilityHostExplicitNotDetected < VagrantError
+      error_key(:capability_host_explicit_not_detected)
+    end
+
+    class CapabilityHostNotDetected < VagrantError
+      error_key(:capability_host_not_detected)
+    end
+
+    class CapabilityInvalid < VagrantError
+      error_key(:capability_invalid)
+    end
+
+    class CapabilityNotFound < VagrantError
+      error_key(:capability_not_found)
+    end
+
     class CFEngineBootstrapFailed < VagrantError
       error_key(:cfengine_bootstrap_failed)
     end
@@ -200,6 +296,10 @@ module Vagrant
       error_key(:command_unavailable_windows)
     end
 
+    class CommunicatorNotFound < VagrantError
+      error_key(:communicator_not_found)
+    end
+
     class ConfigInvalid < VagrantError
       error_key(:config_invalid)
     end
@@ -212,6 +312,10 @@ module Vagrant
       error_key(:copy_private_key_failed)
     end
 
+    class CorruptMachineIndex < VagrantError
+      error_key(:corrupt_machine_index)
+    end
+
     class DarwinNFSMountFailed < VagrantError
       error_key(:darwin_nfs_mount_failed)
     end
@@ -220,10 +324,6 @@ module Vagrant
       error_key(:destroy_requires_force)
     end
 
-    class DotfileIsDirectory < VagrantError
-      error_key(:dotfile_is_directory)
-    end
-
     class DotfileUpgradeJSONError < VagrantError
       error_key(:dotfile_upgrade_json_error)
     end
@@ -236,26 +336,6 @@ module Vagrant
       error_key(:downloader_interrupted)
     end
 
-    class DownloaderFileDoesntExist < VagrantError
-      error_key(:file_missing, "vagrant.downloaders.file")
-    end
-
-    class DownloaderHTTPConnectReset < VagrantError
-      error_key(:connection_reset, "vagrant.downloaders.http")
-    end
-
-    class DownloaderHTTPConnectTimeout < VagrantError
-      error_key(:connection_timeout, "vagrant.downloaders.http")
-    end
-
-    class DownloaderHTTPSocketError < VagrantError
-      error_key(:socket_error, "vagrant.downloaders.http")
-    end
-
-    class DownloaderHTTPStatusError < VagrantError
-      error_key(:status_error, "vagrant.downloaders.http")
-    end
-
     class EnvironmentNonExistentCWD < VagrantError
       error_key(:environment_non_existent_cwd)
     end
@@ -264,18 +344,18 @@ module Vagrant
       error_key(:environment_locked)
     end
 
-    class GemCommandInBundler < VagrantError
-      error_key(:gem_command_in_bundler)
-    end
-
-    class HomeDirectoryMigrationFailed < VagrantError
-      error_key(:home_dir_migration_failed)
+    class HomeDirectoryLaterVersion < VagrantError
+      error_key(:home_dir_later_version)
     end
 
     class HomeDirectoryNotAccessible < VagrantError
       error_key(:home_dir_not_accessible)
     end
 
+    class HomeDirectoryUnknownVersion < VagrantError
+      error_key(:home_dir_unknown_version)
+    end
+
     class ForwardPortAdapterNotFound < VagrantError
       error_key(:forward_port_adapter_not_found)
     end
@@ -288,10 +368,6 @@ module Vagrant
       error_key(:collision_error, "vagrant.actions.vm.forward_ports")
     end
 
-    class ForwardPortCollisionResume < VagrantError
-      error_key(:port_collision_resume)
-    end
-
     class GuestCapabilityInvalid < VagrantError
       error_key(:guest_capability_invalid)
     end
@@ -308,6 +384,10 @@ module Vagrant
       error_key(:guest_not_detected)
     end
 
+    class HostExplicitNotDetected < VagrantError
+      error_key(:host_explicit_not_detected)
+    end
+
     class LinuxMountFailed < VagrantError
       error_key(:linux_mount_failed)
     end
@@ -316,18 +396,26 @@ module Vagrant
       error_key(:linux_nfs_mount_failed)
     end
 
-    class LinuxShellExpandFailed < VagrantError
-      error_key(:linux_shell_expand_failed)
+    class LinuxRDesktopNotFound < VagrantError
+      error_key(:linux_rdesktop_not_found)
     end
 
     class LocalDataDirectoryNotAccessible < VagrantError
       error_key(:local_data_dir_not_accessible)
     end
 
+    class MachineActionLockedError < VagrantError
+      error_key(:machine_action_locked)
+    end
+
     class MachineGuestNotReady < VagrantError
       error_key(:machine_guest_not_ready)
     end
 
+    class MachineLocked < VagrantError
+      error_key(:machine_locked)
+    end
+
     class MachineNotFound < VagrantError
       error_key(:machine_not_found)
     end
@@ -336,26 +424,18 @@ module Vagrant
       error_key(:machine_state_invalid)
     end
 
-    class MultiVMEnvironmentRequired < VagrantError
-      error_key(:multi_vm_required)
-    end
-
     class MultiVMTargetRequired < VagrantError
       error_key(:multi_vm_target_required)
     end
 
-    class NetworkAdapterCollision < VagrantError
-      error_key(:adapter_collision, "vagrant.actions.vm.network")
+    class NetSSHException < VagrantError
+      error_key(:net_ssh_exception)
     end
 
     class NetworkCollision < VagrantError
       error_key(:collides, "vagrant.actions.vm.host_only_network")
     end
 
-    class NetworkNoAdapters < VagrantError
-      error_key(:no_adapters, "vagrant.actions.vm.network")
-    end
-
     class NetworkDHCPAlreadyAttached < VagrantError
       error_key(:dhcp_already_attached, "vagrant.actions.vm.network")
     end
@@ -388,6 +468,14 @@ module Vagrant
       error_key(:nfs_no_valid_ids)
     end
 
+    class NFSNotSupported < VagrantError
+      error_key(:nfs_not_supported)
+    end
+
+    class NFSClientNotInstalledInGuest < VagrantError
+      error_key(:nfs_client_not_installed_in_guest)
+    end
+
     class NoDefaultSyncedFolderImpl < VagrantError
       error_key(:no_default_synced_folder_impl)
     end
@@ -400,6 +488,10 @@ module Vagrant
       error_key(:include_file_missing, "vagrant.actions.general.package")
     end
 
+    class PackageIncludeSymlink < VagrantError
+      error_key(:package_include_symlink)
+    end
+
     class PackageOutputDirectory < VagrantError
       error_key(:output_is_directory, "vagrant.actions.general.package")
     end
@@ -412,50 +504,42 @@ module Vagrant
       error_key(:requires_directory, "vagrant.actions.general.package")
     end
 
-    class PersistDotfileExists < VagrantError
-      error_key(:dotfile_error, "vagrant.actions.vm.persist")
-    end
-
     class ProviderNotFound < VagrantError
       error_key(:provider_not_found)
     end
 
+    class ProviderNotUsable < VagrantError
+      error_key(:provider_not_usable)
+    end
+
     class ProvisionerFlagInvalid < VagrantError
       error_key(:provisioner_flag_invalid)
     end
 
-    class PluginGemError < VagrantError
-      error_key(:plugin_gem_error)
+    class ProvisionerWinRMUnsupported < VagrantError
+      error_key(:provisioner_winrm_unsupported)
     end
 
-    class PluginInstallBadEntryPoint < VagrantError
-      error_key(:plugin_install_bad_entry_point)
+    class PluginGemNotFound < VagrantError
+      error_key(:plugin_gem_not_found)
     end
 
     class PluginInstallLicenseNotFound < VagrantError
       error_key(:plugin_install_license_not_found)
     end
 
-    class PluginInstallNotFound < VagrantError
-      error_key(:plugin_install_not_found)
+    class PluginInstallSpace < VagrantError
+      error_key(:plugin_install_space)
+    end
+
+    class PluginInstallVersionConflict < VagrantError
+      error_key(:plugin_install_version_conflict)
     end
 
     class PluginLoadError < VagrantError
       error_key(:plugin_load_error)
     end
 
-    class PluginLoadFailed < VagrantError
-      error_key(:plugin_load_failed)
-    end
-
-    class PluginLoadFailedWithOutput < VagrantError
-      error_key(:plugin_load_failed_with_output)
-    end
-
-    class PluginNotFound < VagrantError
-      error_key(:plugin_not_found)
-    end
-
     class PluginNotInstalled < VagrantError
       error_key(:plugin_not_installed)
     end
@@ -464,6 +548,22 @@ module Vagrant
       error_key(:plugin_state_file_not_parsable)
     end
 
+    class PluginUninstallSystem < VagrantError
+      error_key(:plugin_uninstall_system)
+    end
+
+    class RSyncError < VagrantError
+      error_key(:rsync_error)
+    end
+
+    class RSyncNotFound < VagrantError
+      error_key(:rsync_not_found)
+    end
+
+    class RSyncNotInstalledInGuest < VagrantError
+      error_key(:rsync_not_installed_in_guest)
+    end
+
     class SCPPermissionDenied < VagrantError
       error_key(:scp_permission_denied)
     end
@@ -476,10 +576,18 @@ module Vagrant
       error_key(:shared_folder_create_failed)
     end
 
+    class ShellExpandFailed < VagrantError
+      error_key(:shell_expand_failed)
+    end
+
     class SSHAuthenticationFailed < VagrantError
       error_key(:ssh_authentication_failed)
     end
 
+    class SSHChannelOpenFail < VagrantError
+      error_key(:ssh_channel_open_fail)
+    end
+
     class SSHConnectEACCES < VagrantError
       error_key(:ssh_connect_eacces)
     end
@@ -504,6 +612,10 @@ module Vagrant
       error_key(:ssh_host_down)
     end
 
+    class SSHInvalidShell< VagrantError
+      error_key(:ssh_invalid_shell)
+    end
+
     class SSHIsPuttyLink < VagrantError
       error_key(:ssh_is_putty_link)
     end
@@ -528,8 +640,8 @@ module Vagrant
       error_key(:ssh_not_ready)
     end
 
-    class SSHPortNotDetected < VagrantError
-      error_key(:ssh_port_not_detected)
+    class SSHRunRequiresKeys < VagrantError
+      error_key(:ssh_run_requires_keys)
     end
 
     class SSHUnavailable < VagrantError
@@ -612,6 +724,10 @@ module Vagrant
       error_key(:virtualbox_install_incomplete)
     end
 
+    class VirtualBoxNoName < VagrantError
+      error_key(:virtualbox_no_name)
+    end
+
     class VMBaseMacNotSpecified < VagrantError
       error_key(:no_base_mac, "vagrant.actions.vm.match_mac")
     end
@@ -628,18 +744,6 @@ module Vagrant
       error_key(:failure, "vagrant.actions.vm.customize")
     end
 
-    class VMFailedToBoot < VagrantError
-      error_key(:failed_to_boot, "vagrant.actions.vm.boot")
-    end
-
-    class VMFailedToRun < VagrantError
-      error_key(:failed_to_run, "vagrant.actions.vm.boot")
-    end
-
-    class VMGuestError < VagrantError
-      error_namespace("vagrant.errors.guest")
-    end
-
     class VMImportFailure < VagrantError
       error_key(:failure, "vagrant.actions.vm.import")
     end
