@@ -11,6 +11,13 @@ module Vagrant
         # @return [Hash<Symbol, Array>]
         attr_reader :action_hooks
 
+        # This contains all the command plugins by name, and returns
+        # the command class and options. The command class is wrapped
+        # in a Proc so that it can be lazy loaded.
+        #
+        # @return [Registry<Symbol, Array<Proc, Hash>>]
+        attr_reader :commands
+
         # This contains all the configuration plugins by scope.
         #
         # @return [Hash<Symbol, Registry>]
@@ -26,12 +33,27 @@ module Vagrant
         # @return [Hash<Symbol, Registry>]
         attr_reader :guest_capabilities
 
+        # This contains all the hosts and their parents.
+        #
+        # @return [Registry<Symbol, Array<Class, Symbol>>]
+        attr_reader :hosts
+
+        # This contains all the registered host capabilities.
+        #
+        # @return [Hash<Symbol, Registry>]
+        attr_reader :host_capabilities
+
         # This contains all the provider plugins by name, and returns
         # the provider class and options.
         #
         # @return [Hash<Symbol, Registry>]
         attr_reader :providers
 
+        # This contains all the registered provider capabilities.
+        #
+        # @return [Hash<Symbol, Registry>]
+        attr_reader :provider_capabilities
+
         # This contains all the synced folder implementations by name.
         #
         # @return [Registry<Symbol, Array<Class, Integer>>]
@@ -41,10 +63,14 @@ module Vagrant
           # The action hooks hash defaults to []
           @action_hooks = Hash.new { |h, k| h[k] = [] }
 
+          @commands = Registry.new
           @configs = Hash.new { |h, k| h[k] = Registry.new }
           @guests  = Registry.new
           @guest_capabilities = Hash.new { |h, k| h[k] = Registry.new }
+          @hosts   = Registry.new
+          @host_capabilities = Hash.new { |h, k| h[k] = Registry.new }
           @providers = Registry.new
+          @provider_capabilities = Hash.new { |h, k| h[k] = Registry.new }
           @synced_folders = Registry.new
         end
       end
