@@ -8,12 +8,10 @@ module Vagrant
       # those plugins as a single unit.
       class Manager
         attr_reader :registered
-        attr_reader :required
 
         def initialize
           @logger = Log4r::Logger.new("vagrant::plugin::v2::manager")
           @registered = []
-          @required = []
         end
 
         # This returns all the action hooks.
@@ -32,11 +30,11 @@ module Vagrant
 
         # This returns all the registered commands.
         #
-        # @return [Hash]
+        # @return [Registry<Symbol, Array<Proc, Hash>>]
         def commands
           Registry.new.tap do |result|
             @registered.each do |plugin|
-              result.merge!(plugin.command)
+              result.merge!(plugin.components.commands)
             end
           end
         end
@@ -89,17 +87,32 @@ module Vagrant
           results
         end
 
-        # This returns all registered host classes.
+        # This returns all the registered guests.
         #
         # @return [Hash]
         def hosts
           Registry.new.tap do |result|
             @registered.each do |plugin|
-              result.merge!(plugin.host)
+              result.merge!(plugin.components.hosts)
             end
           end
         end
 
+        # This returns all the registered host capabilities.
+        #
+        # @return [Hash]
+        def host_capabilities
+          results = Hash.new { |h, k| h[k] = Registry.new }
+
+          @registered.each do |plugin|
+            plugin.components.host_capabilities.each do |host, caps|
+              results[host].merge!(caps)
+            end
+          end
+
+          results
+        end
+
         # This returns all registered providers.
         #
         # @return [Hash]
@@ -111,6 +124,21 @@ module Vagrant
           end
         end
 
+        # This returns all the registered provider capabilities.
+        #
+        # @return [Hash]
+        def provider_capabilities
+          results = Hash.new { |h, k| h[k] = Registry.new }
+
+          @registered.each do |plugin|
+            plugin.components.provider_capabilities.each do |provider, caps|
+              results[provider].merge!(caps)
+            end
+          end
+
+          results
+        end
+
         # This returns all the config classes for the various providers.
         #
         # @return [Hash]
@@ -166,20 +194,10 @@ module Vagrant
           end
         end
 
-        # This registers a required plugin. This should _NEVER_ be called by
-        # the public and should only be called from within Vagrant.
-        def plugin_required(gem_name)
-          if !@required.include?(gem_name)
-            @logger.info("Registered required plugin: #{gem_name}")
-            @required << gem_name
-          end
-        end
-
         # This clears out all the registered plugins. This is only used by
         # unit tests and should not be called directly.
         def reset!
           @registered.clear
-          @required.clear
         end
 
         # This unregisters a plugin so that its components will no longer
