@@ -1,6 +1,9 @@
+require "uri"
+
 require "log4r"
 
 require "vagrant/util/busy"
+require "vagrant/util/platform"
 require "vagrant/util/subprocess"
 
 module Vagrant
@@ -13,15 +16,36 @@ module Vagrant
       # are properly tracked.
       USER_AGENT = "Vagrant/#{VERSION}"
 
+      attr_reader :source
+      attr_reader :destination
+
       def initialize(source, destination, options=nil)
+        options     ||= {}
+
         @logger      = Log4r::Logger.new("vagrant::util::downloader")
         @source      = source.to_s
         @destination = destination.to_s
 
+        begin
+          url = URI.parse(@source)
+          if url.scheme && url.scheme.start_with?("http") && url.user
+            auth = "#{url.user}"
+            auth += ":#{url.password}" if url.password
+            url.user = nil
+            url.password = nil
+            options[:auth] ||= auth
+            @source = url.to_s
+          end
+        rescue URI::InvalidURIError
+          # Ignore, since its clearly not HTTP
+        end
+
         # Get the various optional values
-        options     ||= {}
+        @auth        = options[:auth]
         @ca_cert     = options[:ca_cert]
+        @ca_path     = options[:ca_path]
         @continue    = options[:continue]
+        @headers     = options[:headers]
         @insecure    = options[:insecure]
         @ui          = options[:ui]
         @client_cert = options[:client_cert]
@@ -34,31 +58,10 @@ module Vagrant
       # If this method returns without an exception, the download
       # succeeded. An exception will be raised if the download failed.
       def download!
-        # Build the list of parameters to execute with cURL
-        options = [
-          "--fail",
-          "--location",
-          "--max-redirs", "10",
-          "--user-agent", USER_AGENT,
-          "--output", @destination,
-        ]
-
-        options += ["--cacert", @ca_cert] if @ca_cert
-        options += ["--continue-at", "-"] if @continue
-        options << "--insecure" if @insecure
-        options << "--cert" << @client_cert if @client_cert
+        options, subprocess_options = self.options
+        options += ["--output", @destination]
         options << @source
 
-        # Specify some options for the subprocess
-        subprocess_options = {}
-
-        # If we're in Vagrant, then we use the packaged CA bundle
-        if Vagrant.in_installer?
-          subprocess_options[:env] ||= {}
-          subprocess_options[:env]["CURL_CA_BUNDLE"] =
-            File.expand_path("cacert.pem", ENV["VAGRANT_INSTALLER_EMBEDDED_DIR"])
-        end
-
         # This variable can contain the proc that'll be sent to
         # the subprocess execute.
         data_proc = nil
@@ -109,12 +112,48 @@ module Vagrant
 
               output = "Progress: #{columns[0]}% (Rate: #{columns[11]}/s, Estimated time remaining: #{columns[10]})"
               @ui.clear_line
-              @ui.info(output, :new_line => false)
+              @ui.detail(output, new_line: false)
             end
           end
         end
 
-        # Add the subprocess options onto the options we'll execute with
+        @logger.info("Downloader starting download: ")
+        @logger.info("  -- Source: #{@source}")
+        @logger.info("  -- Destination: #{@destination}")
+
+        begin
+          execute_curl(options, subprocess_options, &data_proc)
+        ensure
+          # If we're outputting to the UI, clear the output to
+          # avoid lingering progress meters.
+          if @ui
+            @ui.clear_line
+
+            # Windows doesn't clear properly for some reason, so we just
+            # output one more newline.
+            @ui.detail("") if Platform.windows?
+          end
+        end
+
+        # Everything succeeded
+        true
+      end
+
+      # Does a HEAD request of the URL and returns the output.
+      def head
+        options, subprocess_options = self.options
+        options.unshift("-I")
+        options << @source
+
+        @logger.info("HEAD: #{@source}")
+        result = execute_curl(options, subprocess_options)
+        result.stdout
+      end
+
+      protected
+
+      def execute_curl(options, subprocess_options, &data_proc)
+        options = options.dup
         options << subprocess_options
 
         # Create the callback that is called if we are interrupted
@@ -124,10 +163,6 @@ module Vagrant
           interrupted = true
         end
 
-        @logger.info("Downloader starting download: ")
-        @logger.info("  -- Source: #{@source}")
-        @logger.info("  -- Destination: #{@destination}")
-
         # Execute!
         result = Busy.busy(int_callback) do
           Subprocess.execute("curl", *options, &data_proc)
@@ -136,21 +171,55 @@ module Vagrant
         # If the download was interrupted, then raise a specific error
         raise Errors::DownloaderInterrupted if interrupted
 
-        # If we're outputting to the UI, clear the output to
-        # avoid lingering progress meters.
-        @ui.clear_line if @ui
-
         # If it didn't exit successfully, we need to parse the data and
         # show an error message.
         if result.exit_code != 0
           @logger.warn("Downloader exit code: #{result.exit_code}")
           parts    = result.stderr.split(/\n*curl:\s+\(\d+\)\s*/, 2)
           parts[1] ||= ""
-          raise Errors::DownloaderError, :message => parts[1].chomp
+          raise Errors::DownloaderError, message: parts[1].chomp
         end
 
-        # Everything succeeded
-        true
+        result
+      end
+
+      # Returns the varoius cURL and subprocess options.
+      #
+      # @return [Array<Array, Hash>]
+      def options
+        # Build the list of parameters to execute with cURL
+        options = [
+          "-q",
+          "--fail",
+          "--location",
+          "--max-redirs", "10",
+          "--user-agent", USER_AGENT,
+        ]
+
+        options += ["--cacert", @ca_cert] if @ca_cert
+        options += ["--capath", @ca_path] if @ca_path
+        options += ["--continue-at", "-"] if @continue
+        options << "--insecure" if @insecure
+        options << "--cert" << @client_cert if @client_cert
+        options << "-u" << @auth if @auth
+
+        if @headers
+          Array(@headers).each do |header|
+            options << "-H" << header
+          end
+        end
+
+        # Specify some options for the subprocess
+        subprocess_options = {}
+
+        # If we're in Vagrant, then we use the packaged CA bundle
+        if Vagrant.in_installer?
+          subprocess_options[:env] ||= {}
+          subprocess_options[:env]["CURL_CA_BUNDLE"] =
+            File.expand_path("cacert.pem", ENV["VAGRANT_INSTALLER_EMBEDDED_DIR"])
+        end
+
+        return [options, subprocess_options]
       end
     end
   end
