@@ -5,6 +5,7 @@ require 'childprocess'
 require "vagrant/util/file_mode"
 require "vagrant/util/platform"
 require "vagrant/util/safe_exec"
+require "vagrant/util/safe_puts"
 require "vagrant/util/subprocess"
 require "vagrant/util/which"
 
@@ -14,6 +15,8 @@ module Vagrant
     # helpers don't depend on any part of Vagrant except what is given
     # via the parameters.
     class SSH
+      extend SafePuts
+
       LOGGER = Log4r::Logger.new("vagrant::util::ssh")
 
       # Checks that the permissions for a private key are valid, and fixes
@@ -30,9 +33,9 @@ module Vagrant
         LOGGER.debug("Checking key permissions: #{key_path}")
         stat = key_path.stat
 
-        if !stat.owned?
-          # The SSH key must be owned by ourselves
-          raise Errors::SSHKeyBadOwner, :key_path => key_path
+        if !stat.owned? && Process.uid != 0
+          # The SSH key must be owned by ourselves, unless we're root
+          raise Errors::SSHKeyBadOwner, key_path: key_path
         end
 
         if FileMode.from_octal(stat.mode) != "600"
@@ -42,13 +45,13 @@ module Vagrant
           # Re-stat the file to get the new mode, and verify it worked
           stat = key_path.stat
           if FileMode.from_octal(stat.mode) != "600"
-            raise Errors::SSHKeyBadPermissions, :key_path => key_path
+            raise Errors::SSHKeyBadPermissions, key_path: key_path
           end
         end
       rescue Errno::EPERM
         # This shouldn't happen since we verify we own the file, but
         # it is possible in theory, so we raise an error.
-        raise Errors::SSHKeyBadPermissions, :key_path => key_path
+        raise Errors::SSHKeyBadPermissions, key_path: key_path
       end
 
       # Halts the running of this process and replaces it with a full-fledged
@@ -67,10 +70,10 @@ module Vagrant
         if !ssh_path
           if Platform.windows?
             raise Errors::SSHUnavailableWindows,
-              :host => ssh_info[:host],
-              :port => ssh_info[:port],
-              :username => ssh_info[:username],
-              :key_path => ssh_info[:private_key_path].join(", ")
+              host: ssh_info[:host],
+              port: ssh_info[:port],
+              username: ssh_info[:username],
+              key_path: ssh_info[:private_key_path].join(", ")
           end
 
           raise Errors::SSHUnavailable
@@ -82,10 +85,10 @@ module Vagrant
           r = Subprocess.execute(ssh_path)
           if r.stdout.include?("PuTTY Link")
             raise Errors::SSHIsPuttyLink,
-              :host => ssh_info[:host],
-              :port => ssh_info[:port],
-              :username => ssh_info[:username],
-              :key_path => ssh_info[:private_key_path].join(", ")
+              host: ssh_info[:host],
+              port: ssh_info[:port],
+              username: ssh_info[:username],
+              key_path: ssh_info[:private_key_path].join(", ")
           end
         end
 
