@@ -2,7 +2,7 @@ module VagrantPlugins
   module GuestCoreOS
     class Guest < Vagrant.plugin("2", :guest)
       def detect?(machine)
-        machine.communicate.test("cat /etc/gentoo-release | grep CoreOS")
+        machine.communicate.test("cat /etc/os-release | grep ID=coreos")
       end
     end
   end
