@@ -20,6 +20,16 @@ module VagrantPlugins
         require_relative "cap/change_host_name"
         Cap::ChangeHostName
       end
+
+      guest_capability("debian", "nfs_client_install") do
+        require_relative "cap/nfs_client"
+        Cap::NFSClient
+      end
+
+      guest_capability("debian", "rsync_install") do
+        require_relative "cap/rsync"
+        Cap::RSync
+      end
     end
   end
 end
