@@ -41,10 +41,15 @@ module VagrantPlugins
           while true
             success = true
 
+            stderr = ""
             mount_commands.each do |command|
               no_such_device = false
+              stderr = ""
               status = machine.communicate.sudo(command, error_check: false) do |type, data|
-                no_such_device = true if type == :stderr && data =~ /No such device/i
+                if type == :stderr
+                  no_such_device = true if data =~ /No such device/i
+                  stderr += data.to_s
+                end
               end
 
               success = status == 0 && !no_such_device
@@ -56,19 +61,23 @@ module VagrantPlugins
             attempts += 1
             if attempts > 10
               raise Vagrant::Errors::LinuxMountFailed,
-                command: mount_commands.join("\n")
+                command: mount_commands.join("\n"),
+                output: stderr
             end
 
-            sleep 2
+            sleep(2*attempts)
           end
 
-          # Chown the directory to the proper user
-          chown_commands = []
-          chown_commands << "chown #{mount_uid}:#{mount_gid} #{expanded_guest_path}"
-          chown_commands << "chown #{mount_uid}:#{mount_gid_old} #{expanded_guest_path}"
+          # Chown the directory to the proper user. We skip this if the
+          # mount options contained a readonly flag, because it won't work.
+          if !options[:mount_options] || !options[:mount_options].include?("ro")
+            chown_commands = []
+            chown_commands << "chown #{mount_uid}:#{mount_gid} #{expanded_guest_path}"
+            chown_commands << "chown #{mount_uid}:#{mount_gid_old} #{expanded_guest_path}"
 
-          exit_status = machine.communicate.sudo(chown_commands[0], error_check: false)
-          machine.communicate.sudo(chown_commands[1]) if exit_status != 0
+            exit_status = machine.communicate.sudo(chown_commands[0], error_check: false)
+            machine.communicate.sudo(chown_commands[1]) if exit_status != 0
+          end
 
           # Emit an upstart event if we can
           if machine.communicate.test("test -x /sbin/initctl")
@@ -76,6 +85,14 @@ module VagrantPlugins
               "/sbin/initctl emit --no-wait vagrant-mounted MOUNTPOINT=#{expanded_guest_path}")
           end
         end
+
+        def self.unmount_virtualbox_shared_folder(machine, guestpath, options)
+          result = machine.communicate.sudo(
+            "umount #{guestpath}", error_check: false)
+          if result == 0
+            machine.communicate.sudo("rmdir #{guestpath}", error_check: false)
+          end
+        end
       end
     end
   end
