@@ -21,10 +21,25 @@ module VagrantPlugins
         Cap::ConfigureNetworks
       end
 
+      guest_capability("redhat", "flavor") do
+        require_relative "cap/flavor"
+        Cap::Flavor
+      end
+
       guest_capability("redhat", "network_scripts_dir") do
         require_relative "cap/network_scripts_dir"
         Cap::NetworkScriptsDir
       end
+
+      guest_capability("redhat", "nfs_client_install") do
+        require_relative "cap/nfs_client"
+        Cap::NFSClient
+      end
+
+      guest_capability("redhat", "rsync_install") do
+        require_relative "cap/rsync"
+        Cap::RSync
+      end
     end
   end
 end
