@@ -35,6 +35,26 @@ module VagrantPlugins
         require_relative "cap/mount_virtualbox_shared_folder"
         Cap::MountVirtualBoxSharedFolder
       end
+
+      guest_capability("solaris", "rsync_installed") do
+        require_relative "cap/rsync"
+        Cap::RSync
+      end
+
+      guest_capability("solaris", "rsync_command") do
+        require_relative "cap/rsync"
+        Cap::RSync
+      end
+
+      guest_capability("solaris", "rsync_post") do
+        require_relative "cap/rsync"
+        Cap::RSync
+      end
+
+      guest_capability("solaris", "rsync_pre") do
+        require_relative "cap/rsync"
+        Cap::RSync
+      end
     end
   end
 end
