@@ -8,6 +8,12 @@ module VagrantPlugins
       # @return [Boolean]
       attr_accessor :auto_nat_dns_proxy
 
+      # If true, will check if guest additions are installed and up to
+      # date. By default, this is true.
+      #
+      # @return [Boolean]
+      attr_accessor :check_guest_additions
+
       # An array of customizations to make on the VM prior to booting it.
       #
       # @return [Array]
@@ -32,6 +38,13 @@ module VagrantPlugins
       # @return [String]
       attr_accessor :name
 
+      # Whether or not this VM has a functional vboxsf filesystem module.
+      # This defaults to true. If you set this to false, then the "virtualbox"
+      # synced folder type won't be valid.
+      #
+      # @return [Boolean]
+      attr_accessor :functional_vboxsf
+
       # The defined network adapters.
       #
       # @return [Hash]
@@ -39,8 +52,10 @@ module VagrantPlugins
 
       def initialize
         @auto_nat_dns_proxy = UNSET_VALUE
+        @check_guest_additions = UNSET_VALUE
         @customizations   = []
         @destroy_unused_network_interfaces = UNSET_VALUE
+        @functional_vboxsf = UNSET_VALUE
         @name             = UNSET_VALUE
         @network_adapters = {}
         @gui              = UNSET_VALUE
@@ -72,8 +87,8 @@ module VagrantPlugins
       #
       # @param [Integer] slot The slot for this network adapter.
       # @param [Symbol] type The type of adapter.
-      def network_adapter(slot, type, *args)
-        @network_adapters[slot] = [type, args]
+      def network_adapter(slot, type, **opts)
+        @network_adapters[slot] = [type, opts]
       end
 
       # Shortcut for setting memory size for the virtual machine.
@@ -84,16 +99,40 @@ module VagrantPlugins
         customize("pre-boot", ["modifyvm", :id, "--memory", size.to_s])
       end
 
+      # Shortcut for setting CPU count for the virtual machine.
+      # Calls #customize internally.
+      #
+      # @param count [Integer, String] the count of CPUs
+      def cpus=(count)
+        customize("pre-boot", ["modifyvm", :id, "--cpus", count.to_i])
+      end
+
+      def merge(other)
+        super.tap do |result|
+          c = customizations.dup
+          c += other.customizations
+          result.instance_variable_set(:@customizations, c)
+        end
+      end
+
       # This is the hook that is called to finalize the object before it
       # is put into use.
       def finalize!
         # Default is to auto the DNS proxy
         @auto_nat_dns_proxy = true if @auto_nat_dns_proxy == UNSET_VALUE
 
+        if @check_guest_additions == UNSET_VALUE
+          @check_guest_additions = true
+        end
+
         if @destroy_unused_network_interfaces == UNSET_VALUE
           @destroy_unused_network_interfaces = false
         end
 
+        if @functional_vboxsf == UNSET_VALUE
+          @functional_vboxsf = true
+        end
+
         # Default is to not show a GUI
         @gui = false if @gui == UNSET_VALUE
 
@@ -102,7 +141,7 @@ module VagrantPlugins
       end
 
       def validate(machine)
-        errors = []
+        errors = _detected_errors
 
         valid_events = ["pre-import", "pre-boot", "post-boot"]
         @customizations.each do |event, _|
