@@ -30,9 +30,10 @@ module VagrantPlugins
 
             # On Windows, we use the VBOX_INSTALL_PATH environmental
             # variable to find VBoxManage.
-            if ENV.has_key?("VBOX_INSTALL_PATH")
+            if ENV.has_key?("VBOX_INSTALL_PATH") ||
+              ENV.has_key?("VBOX_MSI_INSTALL_PATH")
               # Get the path.
-              path = ENV["VBOX_INSTALL_PATH"]
+              path = ENV["VBOX_INSTALL_PATH"] || ENV["VBOX_MSI_INSTALL_PATH"]
               @logger.debug("VBOX_INSTALL_PATH value: #{path}")
 
               # There can actually be multiple paths in here, so we need to
@@ -43,7 +44,7 @@ module VagrantPlugins
 
                 # If the executable exists, then set it as the main path
                 # and break out
-                vboxmanage = "#{path}VBoxManage.exe"
+                vboxmanage = "#{single}VBoxManage.exe"
                 if File.file?(vboxmanage)
                   @vboxmanage_path = Vagrant::Util::Platform.cygwin_windows_path(vboxmanage)
                   break
@@ -95,9 +96,9 @@ module VagrantPlugins
         # The format of each adapter specification should be like so:
         #
         # {
-        #   :type     => :hostonly,
-        #   :hostonly => "vboxnet0",
-        #   :mac_address => "tubes"
+        #   type:     :hostonly,
+        #   hostonly: "vboxnet0",
+        #   mac_address: "tubes"
         # }
         #
         # This must support setting up both host only and bridged networks.
@@ -108,7 +109,7 @@ module VagrantPlugins
 
         # Execute a raw command straight through to VBoxManage.
         #
-        # Accepts a :retryable => true option if the command should be retried
+        # Accepts a retryable: true option if the command should be retried
         # upon failure.
         #
         # Raises a VBoxManage error if it fails.
@@ -132,11 +133,11 @@ module VagrantPlugins
         # The format of each port hash should be the following:
         #
         #     {
-        #       :name => "foo",
-        #       :hostport => 8500,
-        #       :guestport => 80,
-        #       :adapter => 1,
-        #       :protocol => "tcp"
+        #       name: "foo",
+        #       hostport: 8500,
+        #       guestport: 80,
+        #       adapter: 1,
+        #       protocol: "tcp"
         #     }
         #
         # Note that "adapter" and "protocol" are optional and will default
@@ -266,6 +267,10 @@ module VagrantPlugins
         def suspend
         end
 
+        # Unshare folders.
+        def unshare_folders(names)
+        end
+
         # Verifies that the driver is ready to accept work.
         #
         # This should raise a VagrantError if things are not ready.
@@ -297,7 +302,7 @@ module VagrantPlugins
           # Variable to store our execution result
           r = nil
 
-          retryable(:on => Vagrant::Errors::VBoxManageError, :tries => tries, :sleep => 1) do
+          retryable(on: Vagrant::Errors::VBoxManageError, tries: tries, sleep: 1) do
             # If there is an error with VBoxManage, this gets set to true
             errored = false
 
@@ -340,8 +345,8 @@ module VagrantPlugins
             # output.
             if errored
               raise Vagrant::Errors::VBoxManageError,
-                :command => command.inspect,
-                :stderr  => r.stderr
+                command: command.inspect,
+                stderr:  r.stderr
             end
           end
 
@@ -354,11 +359,14 @@ module VagrantPlugins
         def raw(*command, &block)
           int_callback = lambda do
             @interrupted = true
-            @logger.info("Interrupted.")
+
+            # We have to execute this in a thread due to trap contexts
+            # and locks.
+            Thread.new { @logger.info("Interrupted.") }.join
           end
 
           # Append in the options for subprocess
-          command << { :notify => [:stdout, :stderr] }
+          command << { notify: [:stdout, :stderr] }
 
           Vagrant::Util::Busy.busy(int_callback) do
             Vagrant::Util::Subprocess.execute(@vboxmanage_path, *command, &block)
