@@ -26,7 +26,7 @@ module VagrantPlugins
         end
 
         def clear_shared_folders
-          info = execute("showvminfo", @uuid, "--machinereadable", :retryable => true)
+          info = execute("showvminfo", @uuid, "--machinereadable", retryable: true)
           info.split("\n").each do |line|
             if folder = line[/^SharedFolderNameMachineMapping\d+="(.+?)"$/, 1]
               execute("sharedfolder", "remove", @uuid, "--name", folder)
@@ -55,10 +55,10 @@ module VagrantPlugins
 
           # Return the details
           return {
-            :name => name,
-            :ip   => options[:adapter_ip],
-            :netmask => options[:netmask],
-            :dhcp => nil
+            name: name,
+            ip:   options[:adapter_ip],
+            netmask: options[:netmask],
+            dhcp: nil
           }
         end
 
@@ -76,7 +76,7 @@ module VagrantPlugins
 
           execute("list", "vms").split("\n").each do |line|
             if vm = line[/^".+?"\s+\{(.+?)\}$/, 1]
-              info = execute("showvminfo", vm, "--machinereadable", :retryable => true)
+              info = execute("showvminfo", vm, "--machinereadable", retryable: true)
               info.split("\n").each do |line|
                 if adapter = line[/^hostonlyadapter\d+="(.+?)"$/, 1]
                   networks.delete(adapter)
@@ -107,7 +107,7 @@ module VagrantPlugins
 
             if adapter[:bridge]
               args.concat(["--bridgeadapter#{adapter[:adapter]}",
-                          adapter[:bridge]])
+                          adapter[:bridge], "--cableconnected#{adapter[:adapter]}", "on"])
             end
 
             if adapter[:hostonly]
@@ -138,7 +138,6 @@ module VagrantPlugins
         end
 
         def export(path)
-          # TODO: Progress
           execute("export", @uuid, "--output", path.to_s)
         end
 
@@ -215,7 +214,7 @@ module VagrantPlugins
 
           results = []
           current_nic = nil
-          info = execute("showvminfo", uuid, "--machinereadable", :retryable => true)
+          info = execute("showvminfo", uuid, "--machinereadable", retryable: true)
           info.split("\n").each do |line|
             # This is how we find the nic that a FP is attached to,
             # since this comes first.
@@ -263,7 +262,7 @@ module VagrantPlugins
 
         def read_guest_additions_version
           output = execute("guestproperty", "get", @uuid, "/VirtualBox/GuestAdd/Version",
-                           :retryable => true)
+                           retryable: true)
           if value = output[/^Value: (.+?)$/, 1]
             # Split the version by _ since some distro versions modify it
             # to look like this: 4.1.2_ubuntu, and the distro part isn't
@@ -283,13 +282,13 @@ module VagrantPlugins
           if output =~ /^Value: (.+?)$/
             $1.to_s
           else
-            raise Vagrant::Errors::VirtualBoxGuestPropertyNotFound, :guest_property => property
+            raise Vagrant::Errors::VirtualBoxGuestPropertyNotFound, guest_property: property
           end
         end
 
         def read_host_only_interfaces
           dhcp = {}
-          execute("list", "dhcpservers", :retryable => true).split("\n\n").each do |block|
+          execute("list", "dhcpservers", retryable: true).split("\n\n").each do |block|
             info = {}
 
             block.split("\n").each do |line|
@@ -308,7 +307,7 @@ module VagrantPlugins
             dhcp[info[:network]] = info
           end
 
-          execute("list", "hostonlyifs", :retryable => true).split("\n\n").collect do |block|
+          execute("list", "hostonlyifs", retryable: true).split("\n\n").collect do |block|
             info = {}
 
             block.split("\n").each do |line|
@@ -331,7 +330,7 @@ module VagrantPlugins
         end
 
         def read_mac_address
-          info = execute("showvminfo", @uuid, "--machinereadable", :retryable => true)
+          info = execute("showvminfo", @uuid, "--machinereadable", retryable: true)
           info.split("\n").each do |line|
             if mac = line[/^macaddress1="(.+?)"$/, 1]
               return mac
@@ -343,7 +342,7 @@ module VagrantPlugins
 
         def read_mac_addresses
           macs = {}
-          info = execute("showvminfo", @uuid, "--machinereadable", :retryable => true)
+          info = execute("showvminfo", @uuid, "--machinereadable", retryable: true)
           info.split("\n").each do |line|
             if matcher = /^macaddress(\d+)="(.+?)"$/.match(line)
               adapter = matcher[1].to_i
@@ -355,7 +354,7 @@ module VagrantPlugins
         end
 
         def read_machine_folder
-          execute("list", "systemproperties", :retryable => true).split("\n").each do |line|
+          execute("list", "systemproperties", retryable: true).split("\n").each do |line|
             if folder = line[/^Default machine folder:\s+(.+?)$/i, 1]
               return folder
             end
@@ -366,7 +365,7 @@ module VagrantPlugins
 
         def read_network_interfaces
           nics = {}
-          info = execute("showvminfo", @uuid, "--machinereadable", :retryable => true)
+          info = execute("showvminfo", @uuid, "--machinereadable", retryable: true)
           info.split("\n").each do |line|
             if matcher = /^nic(\d+)="(.+?)"$/.match(line)
               adapter = matcher[1].to_i
@@ -393,7 +392,7 @@ module VagrantPlugins
         end
 
         def read_state
-          output = execute("showvminfo", @uuid, "--machinereadable", :retryable => true)
+          output = execute("showvminfo", @uuid, "--machinereadable", retryable: true)
           if output =~ /^name="<inaccessible>"$/
             return :inaccessible
           elsif state = output[/^VMState="(.+?)"$/, 1]
@@ -405,7 +404,7 @@ module VagrantPlugins
 
         def read_used_ports
           ports = []
-          execute("list", "vms", :retryable => true).split("\n").each do |line|
+          execute("list", "vms", retryable: true).split("\n").each do |line|
             if uuid = line[/^".+?" \{(.+?)\}$/, 1]
               # Ignore our own used ports
               next if uuid == @uuid
@@ -421,7 +420,7 @@ module VagrantPlugins
 
         def read_vms
           results = {}
-          execute("list", "vms", :retryable => true).split("\n").each do |line|
+          execute("list", "vms", retryable: true).split("\n").each do |line|
             if line =~ /^"(.+?)" \{(.+?)\}$/
               results[$1.to_s] = $2.to_s
             end
@@ -481,13 +480,36 @@ module VagrantPlugins
           end
 
           # If we reached this point then it didn't work out.
-          raise Vagrant::Errors::VBoxManageError, :command => command.inspect
+          raise Vagrant::Errors::VBoxManageError,
+            command: command.inspect,
+            stderr: r.stderr
         end
 
         def suspend
           execute("controlvm", @uuid, "savestate")
         end
 
+        def unshare_folders(names)
+          names.each do |name|
+            begin
+              execute(
+                "sharedfolder", "remove", @uuid,
+                "--name", name,
+                "--transient")
+
+            execute(
+              "setextradata", @uuid,
+              "VBoxInternal2/SharedFoldersEnableSymlinksCreate/#{name}")
+            rescue Vagrant::Errors::VBoxManageError => e
+              if e.extra_data[:stderr].include?("VBOX_E_FILE_ERROR")
+                # The folder doesn't exist. ignore.
+              else
+                raise
+              end
+            end
+          end
+        end
+
         def verify!
           # This command sometimes fails if kernel drivers aren't properly loaded
           # so we just run the command and verify that it succeeded.
