@@ -1,4 +1,6 @@
 require 'pathname'
+
+require 'vagrant'
 require 'vagrant/util/subprocess'
 
 require File.expand_path("../base", __FILE__)
@@ -20,10 +22,11 @@ module VagrantPlugins
           chown_provisioning_folder
           create_client_key_folder
           upload_validation_key
-          upload_encrypted_data_bag_secret if @config.encrypted_data_bag_secret_key_path
+          upload_encrypted_data_bag_secret
           setup_json
           setup_server_config
           run_chef_client
+          delete_encrypted_data_bag_secret
         end
 
         def cleanup
@@ -32,32 +35,23 @@ module VagrantPlugins
         end
 
         def create_client_key_folder
-          @machine.env.ui.info I18n.t("vagrant.provisioners.chef.client_key_folder")
+          @machine.ui.info I18n.t("vagrant.provisioners.chef.client_key_folder")
           path = Pathname.new(@config.client_key_path)
 
           @machine.communicate.sudo("mkdir -p #{path.dirname}")
         end
 
         def upload_validation_key
-          @machine.env.ui.info I18n.t("vagrant.provisioners.chef.upload_validation_key")
+          @machine.ui.info I18n.t("vagrant.provisioners.chef.upload_validation_key")
           @machine.communicate.upload(validation_key_path, guest_validation_key_path)
         end
 
-        def upload_encrypted_data_bag_secret
-          @machine.env.ui.info I18n.t("vagrant.provisioners.chef.upload_encrypted_data_bag_secret_key")
-          @machine.communicate.upload(encrypted_data_bag_secret_key_path,
-            guest_encrypted_data_bag_secret_key_path)
-        end
-
         def setup_server_config
           setup_config("provisioners/chef_client/client", "client.rb", {
-            :node_name => @config.node_name,
-            :chef_server_url => @config.chef_server_url,
-            :validation_client_name => @config.validation_client_name,
-            :validation_key => guest_validation_key_path,
-            :client_key => @config.client_key_path,
-            :environment => @config.environment,
-            :encrypted_data_bag_secret => guest_encrypted_data_bag_secret_key_path,
+            chef_server_url: @config.chef_server_url,
+            validation_client_name: @config.validation_client_name,
+            validation_key: guest_validation_key_path,
+            client_key: @config.client_key_path,
           })
         end
 
@@ -66,24 +60,28 @@ module VagrantPlugins
             @machine.ui.warn(I18n.t("vagrant.chef_run_list_empty"))
           end
 
-          command_env = @config.binary_env ? "#{@config.binary_env} " : ""
-          command_args = @config.arguments ? " #{@config.arguments}" : ""
-          command = "#{command_env}#{chef_binary_path("chef-client")} " +
-            "-c #{@config.provisioning_path}/client.rb " +
-            "-j #{@config.provisioning_path}/dna.json #{command_args}"
+          if @machine.guest.capability?(:wait_for_reboot)
+            @machine.guest.capability(:wait_for_reboot)
+          end
+
+          command = build_command(:client)
 
           @config.attempts.times do |attempt|
             if attempt == 0
-              @machine.env.ui.info I18n.t("vagrant.provisioners.chef.running_client")
+              @machine.ui.info I18n.t("vagrant.provisioners.chef.running_client")
             else
-              @machine.env.ui.info I18n.t("vagrant.provisioners.chef.running_client_again")
+              @machine.ui.info I18n.t("vagrant.provisioners.chef.running_client_again")
             end
 
-            exit_status = @machine.communicate.sudo(command, :error_check => false) do |type, data|
+            opts = { error_check: false, elevated: true }
+            exit_status = @machine.communicate.sudo(command, opts) do |type, data|
               # Output the data with the proper color based on the stream.
               color = type == :stdout ? :green : :red
-              @machine.env.ui.info(
-                data, :color => color, :new_line => false, :prefix => false)
+
+              data = data.chomp
+              next if data.empty?
+
+              @machine.ui.info(data, color: color)
             end
 
             # There is no need to run Chef again if it converges
@@ -98,33 +96,30 @@ module VagrantPlugins
           File.expand_path(@config.validation_key_path, @machine.env.root_path)
         end
 
-        def encrypted_data_bag_secret_key_path
-          File.expand_path(@config.encrypted_data_bag_secret_key_path, @machine.env.root_path)
-        end
-
-        def guest_encrypted_data_bag_secret_key_path
-          File.join(@config.provisioning_path,
-            "encrypted_data_bag_secret_key.pem")
-        end
-
         def guest_validation_key_path
           File.join(@config.provisioning_path, "validation.pem")
         end
 
         def delete_from_chef_server(deletable)
           node_name = @config.node_name || @machine.config.vm.hostname
-          @machine.env.ui.info(I18n.t(
+          @machine.ui.info(I18n.t(
             "vagrant.provisioners.chef.deleting_from_server",
             deletable: deletable, name: node_name))
 
-          command = ["knife", deletable, "delete", "--yes", node_name]
-          r = Vagrant::Util::Subprocess.execute(*command)
-          if r.exit_code != 0
-            @machine.env.ui.error(I18n.t(
-              "vagrant.chef_client_cleanup_failed",
-              deletable: deletable,
-              stdout: r.stdout,
-              stderr: r.stderr))
+          # Knife is not part of the current Vagrant bundle, so it needs to run
+          # in the context of the system.
+          Vagrant.global_lock do
+            Bundler.with_clean_env do
+              command = ["knife", deletable, "delete", "--yes", node_name]
+              r = Vagrant::Util::Subprocess.execute(*command)
+              if r.exit_code != 0
+                @machine.ui.error(I18n.t(
+                  "vagrant.chef_client_cleanup_failed",
+                  deletable: deletable,
+                  stdout: r.stdout,
+                  stderr: r.stderr))
+              end
+            end
           end
         end
       end
