@@ -2,13 +2,11 @@ require_relative "client"
 require_relative "installer"
 
 module VagrantPlugins
-  module Docker
+  module DockerProvisioner
     class DockerError < Vagrant::Errors::VagrantError
       error_namespace("vagrant.provisioners.docker")
     end
 
-    # TODO: Improve handling of vagrant-lxc specifics (like checking for apparmor
-    #       profile stuff + autocorrection)
     class Provisioner < Vagrant.plugin("2", :provisioner)
       def initialize(machine, config, installer = nil, client = nil)
         super(machine, config)
@@ -32,6 +30,11 @@ module VagrantPlugins
           @client.pull_images(*config.images)
         end
 
+        if config.build_images.any?
+          @machine.ui.info(I18n.t("vagrant.docker_building_images"))
+          @client.build_images(config.build_images)
+        end
+
         if config.containers.any?
           @machine.ui.info(I18n.t("vagrant.docker_starting_containers"))
           @client.run(config.containers)
