@@ -12,9 +12,9 @@ module VagrantPlugins
         attr_accessor :manifests_path
         attr_accessor :module_path
         attr_accessor :options
+        attr_accessor :synced_folder_type
         attr_accessor :temp_dir
         attr_accessor :working_directory
-        attr_accessor :nfs
 
         def initialize
           super
@@ -25,9 +25,27 @@ module VagrantPlugins
           @module_path       = UNSET_VALUE
           @options           = []
           @facter            = {}
+          @synced_folder_type = UNSET_VALUE
           @temp_dir          = UNSET_VALUE
           @working_directory = UNSET_VALUE
-          @nfs               = UNSET_VALUE
+        end
+
+        def nfs=(value)
+          puts "DEPRECATION: The 'nfs' setting for the Puppet provisioner is"
+          puts "deprecated. Please use the 'synced_folder_type' setting instead."
+          puts "The 'nfs' setting will be removed in the next version of Vagrant."
+
+          if value
+            @synced_folder_type = "nfs"
+          else
+            @synced_folder_type = nil
+          end
+        end
+
+        def merge(other)
+          super.tap do |result|
+            result.facter  = @facter.merge(other.facter)
+          end
         end
 
         def finalize!
@@ -46,9 +64,9 @@ module VagrantPlugins
           @hiera_config_path = nil if @hiera_config_path == UNSET_VALUE
           @manifest_file  = "default.pp" if @manifest_file == UNSET_VALUE
           @module_path    = nil if @module_path == UNSET_VALUE
+          @synced_folder_type = nil if @synced_folder_type == UNSET_VALUE
           @temp_dir       = nil if @temp_dir == UNSET_VALUE
           @working_directory = nil if @working_directory == UNSET_VALUE
-          @nfs            = false if @nfs == UNSET_VALUE
 
           # Set a default temp dir that has an increasing counter so
           # that multiple Puppet definitions won't overwrite each other
@@ -84,12 +102,12 @@ module VagrantPlugins
               expand_path(machine.env.root_path)
             if !expanded_path.directory?
               errors << I18n.t("vagrant.provisioners.puppet.manifests_path_missing",
-                               :path => expanded_path.to_s)
+                               path: expanded_path.to_s)
             else
               expanded_manifest_file = expanded_path.join(manifest_file)
-              if !expanded_manifest_file.file?
+              if !expanded_manifest_file.file? && !expanded_manifest_file.directory?
                 errors << I18n.t("vagrant.provisioners.puppet.manifest_missing",
-                                 :manifest => expanded_manifest_file.to_s)
+                                 manifest: expanded_manifest_file.to_s)
               end
             end
           end
@@ -98,7 +116,7 @@ module VagrantPlugins
           this_expanded_module_paths.each do |path|
             if !path.directory?
               errors << I18n.t("vagrant.provisioners.puppet.module_path_missing",
-                               :path => path)
+                               path: path)
             end
           end
 
