@@ -27,8 +27,8 @@ module VagrantPlugins
           end
 
           folder_opts = {}
-          folder_opts[:nfs] = true if @config.nfs
-          folder_opts[:owner] = "root" if !folder_opts[:nfs]
+          folder_opts[:type] = @config.synced_folder_type if @config.synced_folder_type
+          folder_opts[:owner] = "root" if !@config.synced_folder_type
 
           # Share the manifests directory with the guest
           if @config.manifests_path[0].to_sym == :host
@@ -44,6 +44,12 @@ module VagrantPlugins
         end
 
         def provision
+          # If the machine has a wait for reboot functionality, then
+          # do that (primarily Windows)
+          if @machine.guest.capability?(:wait_for_reboot)
+            @machine.guest.capability(:wait_for_reboot)
+          end
+
           # Check that the shared folders are properly shared
           check = []
           if @config.manifests_path[0] == :host
@@ -89,17 +95,26 @@ module VagrantPlugins
         def verify_binary(binary)
           @machine.communicate.sudo(
             "which #{binary}",
-            :error_class => PuppetError,
-            :error_key => :not_detected,
-            :binary => binary)
+            error_class: PuppetError,
+            error_key: :not_detected,
+            binary: binary)
         end
 
         def run_puppet_apply
+          default_module_path = "/etc/puppet/modules"
+          if windows?
+            default_module_path = "/ProgramData/PuppetLabs/puppet/etc/modules"
+          end
+
           options = [config.options].flatten
           module_paths = @module_paths.map { |_, to| to }
           if !@module_paths.empty?
+            # Append the default module path
+            module_paths << default_module_path
+
             # Add the command line switch to add the module path
-            options << "--modulepath '#{module_paths.join(':')}'"
+            module_path_sep = windows? ? ";" : ":"
+            options << "--modulepath '#{module_paths.join(module_path_sep)}'"
           end
 
           if @hiera_config_path
@@ -123,20 +138,35 @@ module VagrantPlugins
               facts << "FACTER_#{key}='#{value}'"
             end
 
+            # If we're on Windows, we need to use the PowerShell style
+            if windows?
+              facts.map! { |v| "`$env:#{v};" }
+            end
+
             facter = "#{facts.join(" ")} "
           end
 
-          command = "#{facter}puppet apply #{options} || [ $? -eq 2 ]"
+          command = "#{facter}puppet apply #{options}"
           if config.working_directory
-            command = "cd #{config.working_directory} && #{command}"
+            if windows?
+              command = "cd #{config.working_directory}; if (`$?) \{ #{command} \}"
+            else
+              command = "cd #{config.working_directory} && #{command}"
+            end
           end
 
-          @machine.env.ui.info I18n.t("vagrant.provisioners.puppet.running_puppet",
-                                      :manifest => config.manifest_file)
+          @machine.ui.info(I18n.t(
+            "vagrant.provisioners.puppet.running_puppet",
+            manifest: config.manifest_file))
 
-          @machine.communicate.sudo(command) do |type, data|
-            if !data.empty?
-              @machine.env.ui.info(data, :new_line => false, :prefix => false)
+          opts = {
+            elevated: true,
+            error_key: :ssh_bad_exit_status_muted,
+            good_exit: [0,2],
+          }
+          @machine.communicate.sudo(command, opts) do |type, data|
+            if !data.chomp.empty?
+              @machine.ui.info(data.chomp)
             end
           end
         end
@@ -149,6 +179,10 @@ module VagrantPlugins
             end
           end
         end
+
+        def windows?
+          @machine.config.vm.communicator == :winrm
+        end
       end
     end
   end
