@@ -7,6 +7,11 @@ module VagrantPlugins
 
       class PuppetServer < Vagrant.plugin("2", :provisioner)
         def provision
+          if @machine.config.vm.communicator == :winrm
+            raise Vagrant::Errors::ProvisionerWinRMUnsupported,
+              name: "puppet_server"
+          end
+
           verify_binary("puppet")
           run_puppet_agent
         end
@@ -14,9 +19,9 @@ module VagrantPlugins
         def verify_binary(binary)
           @machine.communicate.sudo(
             "which #{binary}",
-            :error_class => PuppetServerError,
-            :error_key => :not_detected,
-            :binary => binary)
+            error_class: PuppetServerError,
+            error_key: :not_detected,
+            binary: binary)
         end
 
         def run_puppet_agent
@@ -84,8 +89,8 @@ module VagrantPlugins
 
           @machine.ui.info I18n.t("vagrant.provisioners.puppet_server.running_puppetd")
           @machine.communicate.sudo(command) do |type, data|
-            if !data.empty?
-              @machine.ui.info(data, :new_line => false, :prefix => false)
+            if !data.chomp.empty?
+              @machine.ui.info(data.chomp)
             end
           end
         end
