@@ -29,6 +29,10 @@ module VagrantPlugins
         @privileged  = true if @privileged == UNSET_VALUE
         @binary      = false if @binary == UNSET_VALUE
         @keep_color  = false if @keep_color == UNSET_VALUE
+
+        if @args && args_valid?
+          @args = @args.is_a?(Array) ? @args.map { |a| a.to_s } : @args.to_s
+        end
       end
 
       def validate(machine)
@@ -42,11 +46,20 @@ module VagrantPlugins
         end
 
         # If it is not an URL, we validate the existence of a script to upload
-        if path && ! remote?
+        if path && !remote?
           expanded_path = Pathname.new(path).expand_path(machine.env.root_path)
           if !expanded_path.file?
             errors << I18n.t("vagrant.provisioners.shell.path_invalid",
-                              :path => expanded_path)
+                              path: expanded_path)
+          else
+            data = expanded_path.read(16)
+            if data && !data.valid_encoding?
+              errors << I18n.t(
+                "vagrant.provisioners.shell.invalid_encoding",
+                actual: data.encoding.to_s,
+                default: Encoding.default_external.to_s,
+                path: expanded_path.to_s)
+            end
           end
         end
 
@@ -66,7 +79,15 @@ module VagrantPlugins
       # string or as an array.
       def args_valid?
         return true if !args
-        args.is_a?(String) || args.is_a?(Array)
+        return true if args.is_a?(String)
+        return true if args.is_a?(Fixnum)
+        if args.is_a?(Array)
+          args.each do |a|
+            return false if !a.kind_of?(String) && !a.kind_of?(Fixnum)
+          end
+
+          return true
+        end
       end
 
       def remote?
