@@ -3,19 +3,22 @@ require "vagrant"
 module VagrantPlugins
   module SyncedFolderNFS
     class Config < Vagrant.plugin("2", :config)
+      attr_accessor :functional
       attr_accessor :map_uid
       attr_accessor :map_gid
 
       def initialize
         super
 
-        @map_uid = UNSET_VALUE
-        @map_gid = UNSET_VALUE
+        @functional = UNSET_VALUE
+        @map_uid    = UNSET_VALUE
+        @map_gid    = UNSET_VALUE
       end
 
       def finalize!
-        @map_uid = nil if @map_uid == UNSET_VALUE
-        @map_gid = nil if @map_gid == UNSET_VALUE
+        @functional = true if @functional == UNSET_VALUE
+        @map_uid = :auto if @map_uid == UNSET_VALUE
+        @map_gid = :auto if @map_gid == UNSET_VALUE
       end
 
       def to_s
