@@ -7,17 +7,17 @@ shared_examples "a version 4.x virtualbox driver" do |options|
     it "reads the guest property of the machine referenced by the UUID" do
       key  = "/Foo/Bar"
 
-      subprocess.should_receive(:execute).
+      expect(subprocess).to receive(:execute).
         with("VBoxManage", "guestproperty", "get", uuid, key, an_instance_of(Hash)).
         and_return(subprocess_result(stdout: "Value: Baz\n"))
 
-      subject.read_guest_property(key).should == "Baz"
+      expect(subject.read_guest_property(key)).to eq("Baz")
     end
 
     it "raises a virtualBoxGuestPropertyNotFound exception when the value is not set" do
       key  = "/Not/There"
 
-      subprocess.should_receive(:execute).
+      expect(subprocess).to receive(:execute).
         with("VBoxManage", "guestproperty", "get", uuid, key, an_instance_of(Hash)).
         and_return(subprocess_result(stdout: "No value set!"))
 
@@ -30,13 +30,13 @@ shared_examples "a version 4.x virtualbox driver" do |options|
     it "reads the guest property for the provided adapter number" do
       key = "/VirtualBox/GuestInfo/Net/1/V4/IP"
 
-      subprocess.should_receive(:execute).
+      expect(subprocess).to receive(:execute).
         with("VBoxManage", "guestproperty", "get", uuid, key, an_instance_of(Hash)).
         and_return(subprocess_result(stdout: "Value: 127.1.2.3"))
 
       value = subject.read_guest_ip(1)
 
-      value.should == "127.1.2.3"
+      expect(value).to eq("127.1.2.3")
     end
   end
 end
