@@ -1,10 +1,8 @@
 require File.expand_path("../../../../base", __FILE__)
 
 describe Vagrant::Plugin::V2::Plugin do
-  after(:each) do
-    # We want to make sure that the registered plugins remains empty
-    # after each test.
-    described_class.manager.reset!
+  before do
+    described_class.stub(manager: Vagrant::Plugin::V2::Manager.new)
   end
 
   it "should be able to set and get the name" do
@@ -12,7 +10,7 @@ describe Vagrant::Plugin::V2::Plugin do
       name "foo"
     end
 
-    plugin.name.should == "foo"
+    expect(plugin.name).to eq("foo")
   end
 
   it "should be able to set and get the description" do
@@ -20,7 +18,7 @@ describe Vagrant::Plugin::V2::Plugin do
       description "bar"
     end
 
-    plugin.description.should == "bar"
+    expect(plugin.description).to eq("bar")
   end
 
   describe "action hooks" do
@@ -31,8 +29,8 @@ describe Vagrant::Plugin::V2::Plugin do
 
       hooks_registry = plugin.components.action_hooks
       hooks = hooks_registry[described_class.const_get("ALL_ACTIONS")]
-      hooks.length.should == 1
-      hooks[0].call.should == "bar"
+      expect(hooks.length).to eq(1)
+      expect(hooks[0].call).to eq("bar")
     end
 
     it "should register for a specific action by default" do
@@ -42,8 +40,8 @@ describe Vagrant::Plugin::V2::Plugin do
 
       hooks_registry = plugin.components.action_hooks
       hooks = hooks_registry[:bar]
-      hooks.length.should == 1
-      hooks[0].call.should == "bar"
+      expect(hooks.length).to eq(1)
+      expect(hooks[0].call).to eq("bar")
     end
   end
 
@@ -53,7 +51,28 @@ describe Vagrant::Plugin::V2::Plugin do
         command("foo") { "bar" }
       end
 
-      plugin.command[:foo].should == "bar"
+      expect(plugin.components.commands.keys).to be_include(:foo)
+      expect(plugin.components.commands[:foo][0].call).to eql("bar")
+    end
+
+    it "should register command classes with options" do
+      plugin = Class.new(described_class) do
+        command("foo", opt: :bar) { "bar" }
+      end
+
+      expect(plugin.components.commands.keys).to be_include(:foo)
+      expect(plugin.components.commands[:foo][0].call).to eql("bar")
+      expect(plugin.components.commands[:foo][1][:opt]).to eql(:bar)
+    end
+
+    it "should register commands as primary by default" do
+      plugin = Class.new(described_class) do
+        command("foo") { "bar" }
+        command("bar", primary: false) { "bar" }
+      end
+
+      expect(plugin.components.commands[:foo][1][:primary]).to be_true
+      expect(plugin.components.commands[:bar][1][:primary]).to be_false
     end
 
     ["spaces bad", "sym^bols"].each do |bad|
@@ -79,8 +98,8 @@ describe Vagrant::Plugin::V2::Plugin do
       # Now verify when we actually get the command key that
       # a proper error is raised.
       expect {
-        plugin.command[:foo]
-      }.to raise_error(StandardError)
+        plugin.components.commands[:foo][0].call
+      }.to raise_error(StandardError, "FAIL!")
     end
   end
 
@@ -90,7 +109,7 @@ describe Vagrant::Plugin::V2::Plugin do
         communicator("foo") { "bar" }
       end
 
-      plugin.communicator[:foo].should == "bar"
+      expect(plugin.communicator[:foo]).to eq("bar")
     end
 
     it "should lazily register communicator classes" do
@@ -118,7 +137,7 @@ describe Vagrant::Plugin::V2::Plugin do
         config("foo") { "bar" }
       end
 
-      plugin.components.configs[:top][:foo].should == "bar"
+      expect(plugin.components.configs[:top][:foo]).to eq("bar")
     end
 
     it "should lazily register configuration classes" do
@@ -144,7 +163,7 @@ describe Vagrant::Plugin::V2::Plugin do
         config("foo", :provider) { "bar" }
       end
 
-      plugin.components.configs[:provider][:foo].should == "bar"
+      expect(plugin.components.configs[:provider][:foo]).to eq("bar")
     end
   end
 
@@ -154,7 +173,7 @@ describe Vagrant::Plugin::V2::Plugin do
         guest("foo") { "bar" }
       end
 
-      plugin.components.guests[:foo].should == ["bar", nil]
+      expect(plugin.components.guests[:foo]).to eq(["bar", nil])
     end
 
     it "should lazily register guest classes" do
@@ -182,7 +201,7 @@ describe Vagrant::Plugin::V2::Plugin do
         guest_capability("foo", "bar") { "baz" }
       end
 
-      plugin.components.guest_capabilities[:foo][:bar].should == "baz"
+      expect(plugin.components.guest_capabilities[:foo][:bar]).to eq("baz")
     end
   end
 
@@ -192,7 +211,7 @@ describe Vagrant::Plugin::V2::Plugin do
         host("foo") { "bar" }
       end
 
-      plugin.host[:foo].should == "bar"
+      expect(plugin.components.hosts[:foo]).to eq(["bar", nil])
     end
 
     it "should lazily register host classes" do
@@ -214,13 +233,25 @@ describe Vagrant::Plugin::V2::Plugin do
     end
   end
 
+  describe "host capabilities" do
+    it "should register host capabilities" do
+      plugin = Class.new(described_class) do
+        host_capability("foo", "bar") { "baz" }
+      end
+
+      expect(plugin.components.host_capabilities[:foo][:bar]).to eq("baz")
+    end
+  end
+
   describe "providers" do
     it "should register provider classes" do
       plugin = Class.new(described_class) do
         provider("foo") { "bar" }
       end
 
-      plugin.components.providers[:foo].should == ["bar", {}]
+      result = plugin.components.providers[:foo]
+      expect(result[0]).to eq("bar")
+      expect(result[1][:priority]).to eq(5)
     end
 
     it "should register provider classes with options" do
@@ -228,7 +259,10 @@ describe Vagrant::Plugin::V2::Plugin do
         provider("foo", foo: "yep") { "bar" }
       end
 
-      plugin.components.providers[:foo].should == ["bar", { foo: "yep" }]
+      result = plugin.components.providers[:foo]
+      expect(result[0]).to eq("bar")
+      expect(result[1][:priority]).to eq(5)
+      expect(result[1][:foo]).to eq("yep")
     end
 
     it "should lazily register provider classes" do
@@ -250,13 +284,23 @@ describe Vagrant::Plugin::V2::Plugin do
     end
   end
 
+  describe "provider capabilities" do
+    it "should register host capabilities" do
+      plugin = Class.new(described_class) do
+        provider_capability("foo", "bar") { "baz" }
+      end
+
+      expect(plugin.components.provider_capabilities[:foo][:bar]).to eq("baz")
+    end
+  end
+
   describe "provisioners" do
     it "should register provisioner classes" do
       plugin = Class.new(described_class) do
         provisioner("foo") { "bar" }
       end
 
-      plugin.provisioner[:foo].should == "bar"
+      expect(plugin.provisioner[:foo]).to eq("bar")
     end
 
     it "should lazily register provisioner classes" do
@@ -284,7 +328,7 @@ describe Vagrant::Plugin::V2::Plugin do
         synced_folder("foo") { "bar" }
       end
 
-      plugin.components.synced_folders[:foo].should == ["bar", 10]
+      expect(plugin.components.synced_folders[:foo]).to eq(["bar", 10])
     end
 
     it "should be able to specify priorities" do
@@ -292,7 +336,7 @@ describe Vagrant::Plugin::V2::Plugin do
         synced_folder("foo", 50) { "bar" }
       end
 
-      plugin.components.synced_folders[:foo].should == ["bar", 50]
+      expect(plugin.components.synced_folders[:foo]).to eq(["bar", 50])
     end
 
     it "should lazily register implementations" do
@@ -318,7 +362,7 @@ describe Vagrant::Plugin::V2::Plugin do
     let(:manager) { described_class.manager }
 
     it "should have no registered plugins" do
-      manager.registered.should be_empty
+      expect(manager.registered).to be_empty
     end
 
     it "should register a plugin when a name is set" do
@@ -326,7 +370,7 @@ describe Vagrant::Plugin::V2::Plugin do
         name "foo"
       end
 
-      manager.registered.should == [plugin]
+      expect(manager.registered).to eq([plugin])
     end
 
     it "should register a plugin only once" do
@@ -335,7 +379,7 @@ describe Vagrant::Plugin::V2::Plugin do
         name "bar"
       end
 
-      manager.registered.should == [plugin]
+      expect(manager.registered).to eq([plugin])
     end
   end
 end
